/* 
 * NewsBlob - Live RSS news reader.
 * Copyright 2009 Lars Torger Høye.
 *
 * This file is part of NewsBlob.
 *
 * NewsBlob is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * NewsBlob is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with NewsBlob.  If not, see <http://www.gnu.org/licenses/>.
 */

package newsblob.feed;

import java.awt.Menu;
import newsblob.blob.Blob;
import java.awt.MenuItem;
import java.text.DecimalFormat;
import java.util.Calendar;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import newsblob.NewsBlobMenu;
import newsblob.blob.LinkListener;
import newsblob.feed.article.Article;
import newsblob.feed.article.ArticleBuffer;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

/**
 *
 * @author Lars Torger Høye
 */
public class NewsfeedThread extends Thread {
    
    private Newsfeed m_newsfeed;
    private Menu m_menu;
    private ArticleBuffer m_articles = new ArticleBuffer();
    private Blob m_output;

    private boolean m_closeStream;

    private DocumentBuilder m_documentBuilder;
    private XPath m_xPath;
    private XPathExpression m_xPathExpression;

    private long m_updateTimestamp;

    public NewsfeedThread(Newsfeed newsfeed, Blob output) throws ParserConfigurationException, XPathExpressionException {
        m_newsfeed = newsfeed;
        m_output = output;
        
        m_closeStream = false;

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        m_documentBuilder = factory.newDocumentBuilder();

        m_xPath = XPathFactory.newInstance().newXPath();
        m_xPathExpression = m_xPath.compile("/rss/channel//item");

        m_menu = new Menu(m_newsfeed.getTitle());
        createMenu();
        NewsBlobMenu.getInstance().add(m_menu);
    }

    private MenuItem createMenu() {
        MenuItem item = new MenuItem("Disable");
        item.setEnabled(false);  //not implemented yet
        //item.addActionListener(new NewsfeedMenuListener(newsBlob, m_newsfeed));
        m_menu.add(item);

        item = new MenuItem("Remove");
        item.addActionListener(new RemoveNewsfeedListener(m_newsfeed));
        m_menu.add(item);
        m_menu.addSeparator();

        item = new MenuItem("Settings...");
        item.setEnabled(false);  //not implemented yet
        //item.addActionListener(new NewsfeedMenuListener(newsBlob, m_newsfeed));
        m_menu.add(item);

        return m_menu;
    }

    private void addMenuItem(Article article) {
        DecimalFormat format = new DecimalFormat("00");
        MenuItem item = new MenuItem(
                format.format(Calendar.getInstance().get(Calendar.HOUR_OF_DAY))+":"
                +format.format(Calendar.getInstance().get(Calendar.MINUTE))
                +" - "+article.getTitle());
        item.addActionListener(new LinkListener(article.getLink()));
        if(m_menu.getItemCount() > 10+4)
            m_menu.remove(14);
        else if(m_menu.getItemCount() == 4)
            m_menu.addSeparator();
        m_menu.insert(item, 5);
    }

    public Newsfeed getNewsfeed() {
        return m_newsfeed;
    }

    @Override
    public void run() {

        boolean firstRun = true;

        while(!m_closeStream) {
            try {
                m_updateTimestamp = System.currentTimeMillis();
                Document doc = m_documentBuilder.parse(m_newsfeed.getUniqueURI());
                NodeList result = (NodeList)m_xPathExpression.evaluate(doc, XPathConstants.NODESET);

                for (int i = result.getLength()-1; i > -1  && !m_closeStream; i--) {
                    NodeList children = result.item(i).getChildNodes();
                    int title = 0;
                    int description = 1;
                    int link = 2;
                    for (int j = 0; j < children.getLength(); j++) {
                        if("title".equals(children.item(j).getNodeName()))
                                title = j;
                        else if("description".equals(children.item(j).getNodeName()))
                                description = j;
                        else if("link".equals(children.item(j).getNodeName()))
                                link = j;
                    }

                    Article article = new Article(children.item(title).getTextContent(), children.item(description).getTextContent(), children.item(link).getTextContent());
                    if(!m_articles.contains(article)) {
                        addMenuItem(article);
                        if(!firstRun)
                            m_output.print(m_newsfeed, article);
                        m_articles.addArticle(article);
                    }
                }
                m_documentBuilder.reset();
                long sleepTime = m_updateTimestamp + (m_newsfeed.getUpdateFrequenzy()*60000) - System.currentTimeMillis();
                if(sleepTime > 0 && !m_closeStream) {
                    Thread.sleep(sleepTime);
                }
                firstRun = false;
            }
            catch(Exception e) {}
        }
    }
    
    public void close() {
        m_closeStream = true;
        //this.interrupt();
        NewsBlobMenu.getInstance().remove(m_menu);
    }
}
